\ DS1302 Driver, DS1302mini.fth
\ V0.2  dp 160802.2100
\ DS1302 inexpensive single modules from ebay.
\ pull CCE to ground with 2.2K to prevent floating during startup 
\ this driver is specifically for setting the TACHYON RTC routines from a DS1302 module or chip at system boot.
\ using check pin assignments, CHANGE THIS AS NEEDED FOR THE PROJECT

\ 160802 dp, updated module to allow for setting the day of the week   DSDAY!  ( dayofweek -- )  1 = mon ... 

TACHYON

FORGET DS1302mini.fth
pub    DS1302mini.fth  ." DS1302 RTC Mini 160802.2100 " ;

DECIMAL
[~ 

{
  \ 
  \ Routines based on SPIN Example
  \
}

\ DS1302 Command codes, minimal 
%10000000 CONSTANT wrtsecreg   --- OR with 1 to get respective read register
%10000010 CONSTANT wrtminreg
%10000100 CONSTANT wrthrsreg
%10000110 CONSTANT wrtdatereg
%10001010 CONSTANT wrtdayreg
%10001000 CONSTANT wrtmonreg
%10001100 CONSTANT wrtyrreg
%10001110 CONSTANT wrtctrlreg
%10010000 CONSTANT wrttrklreg
%10111111 CONSTANT rdbrstreg   --- read clock in burst mode
%11111111 CONSTANT rdramreg    --- read and write ram bytes
%11111110 CONSTANT wrtramreg

   ---  SPI-ish pins
#P0 |< CONSTANT CCLK    --- eeprom CCLK     
#P1 |< CONSTANT CSDA    --- eeprom data
#P14 |< CONSTANT CCE     --- pull to ground with 2.2K 


{
   --- quick start board
#P6 |< CONSTANT CCLK    --- eeprom CCLK     
#P8 |< CONSTANT CSDA    --- eeprom data
#P10 |< CONSTANT CCE     --- pull to ground with 2.2K 
}
{ write data to the ds1302 }
pub  DS1302! ( data -- )


	CSDA OUTPUTS     \ set CSDA PIN to OUTPUT (redundant)
	CSDA SWAP        \ set up the CSDA pin on DS1302 (pinmask data -- )
	CCLK OUTCLR      \ force CCLK lo
	8 FOR SHROUT
		CCLK OUTSET
		NOP
		CCLK OUTCLR
	NEXT
	2DROP      \ drop the iomask and data

;

{ read data from the ds1302 }
pub  DS1302@ ( -- data )
	CSDA DUP INPUTS           \ set  CSDA PIN to INPUT
	0  ( mask data -- )
	8 FOR SHRINP
		CCLK OUTSET
		NOP
		CCLK OUTCLR
	NEXT
	NIP
;

{ DS1302 set register words }
pub SET_DS ( packed reg -- )   --- Set the DS register
	CCE OUTSET
	DS1302!
	DS1302!
	CCE OUTCLR
;

pub SET_DS0 ( packed reg -- )   --- Set the DS register, second byte 0
	CCE OUTSET
	DS1302!
	0 DS1302!
	CCE OUTCLR
;


pub DSDAY! ( day -- )     --- day 1 = mon .... 7 = sun
     wrtdayreg SET_DS
;
    


pub  INIT
	CCLK OUTCLR
	CCE  OUTCLR
	#100 ms

	\ Clear write-protect bit, undefined at start up
	wrtctrlreg SET_DS0
        
	\ Clear halt bit, undefined on startup
	wrtsecreg SET_DS0

	\ Set 24hr mode, clear bit 7 of hours register
	wrthrsreg SET_DS0

	\ Clear trickle charger settings
	wrttrklreg SET_DS0
;

{ binary coded decimal to decimal , single byte }
pub B2D ( bcd -- decimal ) 
	DUP %11110000 AND 4 SHR #10 * SWAP %00001111 AND + 
;

{ decimal to binary coded decimal , single byte }
pub D2B ( decimal -- bcd )
	#10 U/MOD 4 SHL + 
;


{ get the ds1302 battery backed ram from a buffer, 31 bytes max }
pub DSGETRAM  ( buffer CNT --  )  --- return data into buffer
	CCE OUTSET                          
	rdramreg DS1302!  --- read ram in burst mode
	ADO     
		DS1302@
		#24 SHR I C!        --- write the byte to the buffer
	LOOP
	CCE OUTCLR
;

{ set the ds1302 battery backed ram from a buffer, 31 bytes max }
pub DSSETRAM  ( buffer CNT --  )  --- write data into buffer
	CCE OUTSET                          
	wrtramreg DS1302!  --- read ram in burst mode
	ADO
		I C@       --- get the buffer data  
		DS1302!
	LOOP
	CCE OUTCLR
;


{ get the time from the RTC and set the TACHYON Kernel RTC routines }
pub SETDST  ( -- year day month hrs date mins sec ) 
	CCE OUTSET                          
	rdbrstreg DS1302!  --- read in burst mode
	7 FOR 
		DS1302@
		#24 SHR
	NEXT
	CCE OUTCLR
  
	SWAP DAY  --- day   
	B2D  #10000 *    --- yy 
	SWAP B2D #100 * +     --- mm 
	SWAP B2D  +            --- dd 
  
	DUP CR ." Setting Date: " .DEC
	DATE!    --- setting the kernel date
  
	B2D  #10000 *  --- hh
	SWAP B2D #100 *  + --- mm
	SWAP B2D  +         --- mins

	DUP CR ." Setting Time: " .DEC
	TIME!    --- set the kernel rtc
  
	DROP    --- eighth read from the DS1302
;

{ set the ds1302 date using #yymmss format }
pub DSDATE! ( #yymmdd -- )
	#100 U/MOD #100 U/MOD 
	#10 U/MOD   --- yy
	4 SHL OR
	wrtyrreg SET_DS    --- set year
	#10 U/MOD  --- mm
	4 SHL OR
	wrtmonreg SET_DS    --- set month
	#10 U/MOD  --- dd
	4 SHL OR
	wrtdatereg SET_DS     --- set date
;	

{ set the ds1302 time using #hhmmss format }
pub DSTIME! ( #hhmmss -- )
	#100 U/MOD #100 U/MOD 
	#10 U/MOD   --- hh
	4 SHL OR
	wrthrsreg SET_DS   --- set hours
	#10 U/MOD  --- mm
	4 SHL OR
	wrtminreg SET_DS   --- set minutes
	#10 U/MOD  --- dd
	4 SHL OR
	wrtsecreg SET_DS   --- set seconds
;


{ ******* usage  ********

--- set the ds1302 date to 04/20/13
#150420 DSDATE!

--- set the ds1302 time to 11:11:11    24hr time
#111111 DSTIME!

--- set the day of the week  1 = Monday ... 7 = Sunday
#1  DSDAY!    

--- set TACHYON RTC from the DS1302
SETDST

--- get TACHYON time, setter is TIME! in #hhmmss format
TIME@

--- get TACHYON date time
.DT

--- get TACHYON date, setter is DATE! in #yymmdd format
DATE@

--- write 31 bytes to ds1302 ram
#31 BYTES buffer1
#31 BYTES buffer2

0 buffer1 #31 ADO 1+ DUP I C! LOOP DROP   --- load the buffer with 1 - 31
 
buffer1 #31 DSSETRAM    --- set the ds1302 ram from buffer1  #31 bytes

--- read 31 bytes from ds1302 ram
#31 BYTES buffer2

buffer2 #31 DSGETRAM      --- get the ds1302 ram into buffer2  #31 bytes

buffer2 #31 ADO CR I C@ . LOOP   --- read buffer2 and print 

******** ********* } 
	
]~
	  
END




